@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import os
 import shutil
 from typing import (
@@ -8,16 +10,15 @@ from typing import (
     KeysView,
     List,
     Mapping,
+    MutableMapping,
     Optional,
-    Tuple,
-    Union,
+    Sequence,
     cast,
 )
 
-import attr
-import fs
 import fs.base
 import fs.tempfs
+from attr import define, field
 from fontTools.ufoLib import UFOFileStructure, UFOReader, UFOWriter
 
 from ufoLib2.constants import DEFAULT_LAYER_NAME
@@ -27,8 +28,10 @@ from ufoLib2.objects.glyph import Glyph
 from ufoLib2.objects.guideline import Guideline
 from ufoLib2.objects.imageSet import ImageSet
 from ufoLib2.objects.info import Info
+from ufoLib2.objects.kerning import Kerning, KerningPair
 from ufoLib2.objects.layer import Layer
 from ufoLib2.objects.layerSet import LayerSet
+from ufoLib2.objects.lib import Lib, _convert_Lib, _get_lib, _set_lib
 from ufoLib2.objects.misc import (
     BoundingBox,
     _deepcopy_unlazify_attrs,
@@ -38,23 +41,39 @@ from ufoLib2.objects.misc import (
 from ufoLib2.typing import HasIdentifier, PathLike, T
 
 
-def _convert_Info(value: Union[Info, Mapping[str, Any]]) -> Info:
+def _convert_LayerSet(value: LayerSet | Iterable[Layer]) -> LayerSet:
+    return value if isinstance(value, LayerSet) else LayerSet.from_iterable(value)
+
+
+def _convert_Info(value: Info | Mapping[str, Any]) -> Info:
     return value if isinstance(value, Info) else Info(**value)
 
 
-def _convert_DataSet(value: Union[DataSet, Mapping[str, Any]]) -> DataSet:
-    return value if isinstance(value, DataSet) else DataSet(**value)
+def _convert_DataSet(value: DataSet | MutableMapping[str, bytes]) -> DataSet:
+    return value if isinstance(value, DataSet) else DataSet(value)  # type: ignore
 
 
-def _convert_ImageSet(value: Union[ImageSet, Mapping[str, Any]]) -> ImageSet:
-    return value if isinstance(value, ImageSet) else ImageSet(**value)
+def _convert_ImageSet(value: ImageSet | MutableMapping[str, bytes]) -> ImageSet:
+    return value if isinstance(value, ImageSet) else ImageSet(value)  # type: ignore
 
 
-def _convert_Features(value: Union[Features, str]) -> Features:
+def _convert_Features(value: Features | str) -> Features:
     return value if isinstance(value, Features) else Features(value)
 
 
-@attr.s(auto_attribs=True, slots=True, repr=False, eq=False)
+def _convert_Kerning(value: Mapping[KerningPair, float]) -> Kerning:
+    return value if isinstance(value, Kerning) else Kerning(value)
+
+
+def _get_kerning(self: Font) -> Kerning:
+    return self._kerning
+
+
+def _set_kerning(self: Font, value: Mapping[KerningPair, float]) -> None:
+    self._kerning = _convert_Kerning(value)
+
+
+@define(kw_only=True)
 class Font:
     """A data class representing a single Unified Font Object (UFO).
 
@@ -67,9 +86,6 @@ class Font:
     http://unifiedfontobject.org/versions/ufo3/index.html.
 
     Parameters:
-        path: The path to the UFO to load. The only positional parameter and a
-            defcon-API-compatibility parameter. We recommend to use the
-            :meth:`.Font.open` class method instead.
         layers (LayerSet): A mapping of layer names to Layer objects.
         info (Info): The font Info object.
         features (Features): The font Features object.
@@ -117,68 +133,47 @@ class Font:
         default layer.
     """
 
-    _path: Optional[PathLike] = attr.ib(default=None, metadata=dict(copyable=False))
-
-    layers: LayerSet = attr.ib(
+    layers: LayerSet = field(
         factory=LayerSet.default,
-        validator=attr.validators.instance_of(LayerSet),
-        kw_only=True,
+        converter=_convert_LayerSet,
+        metadata={"omit_if_default": False},
     )
     """LayerSet: A mapping of layer names to Layer objects."""
 
-    info: Info = attr.ib(factory=Info, converter=_convert_Info, kw_only=True)
+    info: Info = field(factory=Info, converter=_convert_Info)
     """Info: The font Info object."""
 
-    features: Features = attr.ib(
-        factory=Features, converter=_convert_Features, kw_only=True
-    )
+    features: Features = field(factory=Features, converter=_convert_Features)
     """Features: The font Features object."""
 
-    groups: Dict[str, List[str]] = attr.ib(factory=dict, kw_only=True)
+    groups: Dict[str, List[str]] = field(factory=dict)
     """Dict[str, List[str]]: A mapping of group names to a list of glyph names."""
 
-    kerning: Dict[Tuple[str, str], float] = attr.ib(factory=dict, kw_only=True)
+    _kerning: Kerning = field(factory=Kerning, converter=_convert_Kerning)
     """Dict[Tuple[str, str], float]: A mapping of a tuple of first and second kerning
     pair to a kerning value."""
 
-    lib: Dict[str, Any] = attr.ib(factory=dict, kw_only=True)
-    """Dict[str, Any]: A mapping of keys to arbitrary values."""
+    _lib: Lib = field(factory=Lib, converter=_convert_Lib)
+    """Dict[str, PlistEncodable]: A mapping of keys to arbitrary plist values."""
 
-    data: DataSet = attr.ib(factory=DataSet, converter=_convert_DataSet, kw_only=True)
+    data: DataSet = field(factory=DataSet, converter=_convert_DataSet)
     """DataSet: A mapping of data file paths to arbitrary data."""
 
-    images: ImageSet = attr.ib(
-        factory=ImageSet, converter=_convert_ImageSet, kw_only=True
-    )
+    images: ImageSet = field(factory=ImageSet, converter=_convert_ImageSet)
     """ImageSet: A mapping of image file paths to arbitrary image data."""
 
-    _lazy: Optional[bool] = attr.ib(default=None, kw_only=True)
-    _validate: bool = attr.ib(default=True, kw_only=True)
-
-    _reader: Optional[UFOReader] = attr.ib(default=None, kw_only=True, init=False)
-    _fileStructure: Optional[UFOFileStructure] = attr.ib(default=None, init=False)
-
-    def __attrs_post_init__(self) -> None:
-        if self._path is not None:
-            # if lazy argument is not set, default to lazy=True if path is provided
-            if self._lazy is None:
-                self._lazy = True
-            reader = UFOReader(self._path, validate=self._validate)
-            self.layers = LayerSet.read(reader, lazy=self._lazy)
-            self.data = DataSet.read(reader, lazy=self._lazy)
-            self.images = ImageSet.read(reader, lazy=self._lazy)
-            self.info = Info.read(reader)
-            self.features = Features(reader.readFeatures())
-            self.groups = reader.readGroups()
-            self.kerning = reader.readKerning()
-            self.lib = reader.readLib()
-            self._fileStructure = reader.fileStructure
-            if self._lazy:
-                # keep the reader around so we can close it when done
-                self._reader = reader
+    # init=False args, set by alternate open/read classmethod constructors
+    _path: Optional[PathLike] = field(
+        default=None, metadata=dict(copyable=False), eq=False, init=False
+    )
+    _lazy: Optional[bool] = field(default=None, init=False, eq=False)
+    _reader: Optional[UFOReader] = field(default=None, init=False, eq=False)
+    _fileStructure: Optional[UFOFileStructure] = field(
+        default=None, init=False, eq=False
+    )
 
     @classmethod
-    def open(cls, path: PathLike, lazy: bool = True, validate: bool = True) -> "Font":
+    def open(cls, path: PathLike, lazy: bool = True, validate: bool = True) -> Font:
         """Instantiates a new Font object from a path to a UFO.
 
         Args:
@@ -196,7 +191,7 @@ class Font:
         return self
 
     @classmethod
-    def read(cls, reader: UFOReader, lazy: bool = True) -> "Font":
+    def read(cls, reader: UFOReader, lazy: bool = True) -> Font:
         """Instantiates a Font object from a :class:`fontTools.ufoLib.UFOReader`.
 
         Args:
@@ -211,10 +206,10 @@ class Font:
             info=Info.read(reader),
             features=Features(reader.readFeatures()),
             groups=reader.readGroups(),
-            kerning=reader.readKerning(),
-            lib=reader.readLib(),
-            lazy=lazy,
+            kerning=Kerning(reader.readKerning()),
+            lib=Lib(reader.readLib()),
         )
+        self._lazy = lazy
         self._fileStructure = reader.fileStructure
         if lazy:
             # keep the reader around so we can close it when done
@@ -222,31 +217,31 @@ class Font:
         return self
 
     def __contains__(self, name: object) -> bool:
-        return name in self.layers.defaultLayer
+        return name in self.layers._defaultLayer
 
     def __delitem__(self, name: str) -> None:
-        del self.layers.defaultLayer[name]
+        del self.layers._defaultLayer[name]
 
     def __getitem__(self, name: str) -> Glyph:
-        return self.layers.defaultLayer[name]
+        return self.layers._defaultLayer[name]
 
     def __setitem__(self, name: str, glyph: Glyph) -> None:
-        self.layers.defaultLayer[name] = glyph
+        self.layers._defaultLayer[name] = glyph
 
     def __iter__(self) -> Iterator[Glyph]:
-        return iter(self.layers.defaultLayer)
+        return iter(self.layers._defaultLayer)
 
     def __len__(self) -> int:
-        return len(self.layers.defaultLayer)
+        return len(self.layers._defaultLayer)
 
-    def get(self, name: str, default: Optional[T] = None) -> Union[Optional[T], Glyph]:
+    def get(self, name: str, default: T | None = None) -> T | Glyph | None:
         """Return the :class:`.Glyph` object for name if it is present in the
         default layer, otherwise return ``default``."""
-        return self.layers.defaultLayer.get(name, default)
+        return self.layers._defaultLayer.get(name, default)
 
     def keys(self) -> KeysView[str]:
         """Return a list of glyph names in the default layer."""
-        return self.layers.defaultLayer.keys()
+        return self.layers._defaultLayer.keys()
 
     def close(self) -> None:
         """Closes the UFOReader if it still exists to finalize any outstanding
@@ -254,7 +249,7 @@ class Font:
         if self._reader is not None:
             self._reader.close()
 
-    def __enter__(self) -> "Font":
+    def __enter__(self) -> Font:
         # TODO: Document an example for this.
         return self
 
@@ -307,7 +302,7 @@ class Font:
         return not result
 
     @property
-    def reader(self) -> Optional[UFOReader]:
+    def reader(self) -> UFOReader | None:
         """Returns the underlying :class:`fontTools.ufoLib.UFOReader`."""
         return self._reader
 
@@ -324,7 +319,7 @@ class Font:
     __deepcopy__ = _deepcopy_unlazify_attrs
 
     @property
-    def glyphOrder(self) -> List[str]:
+    def glyphOrder(self) -> list[str]:
         """The font's glyph order.
 
         See http://unifiedfontobject.org/versions/ufo3/lib.plist/#publicglyphorder for
@@ -341,10 +336,10 @@ class Font:
             Sets the font's glyph order. If ``value`` is None or an empty list, the
             glyph order key will be deleted from the lib if it exists.
         """
-        return list(self.lib.get("public.glyphOrder", []))
+        return list(cast(Sequence[str], self.lib.get("public.glyphOrder", [])))
 
     @glyphOrder.setter
-    def glyphOrder(self, value: Optional[List[str]]) -> None:
+    def glyphOrder(self, value: list[str] | None) -> None:
         if value is None or len(value) == 0:
             if "public.glyphOrder" in self.lib:
                 del self.lib["public.glyphOrder"]
@@ -352,7 +347,7 @@ class Font:
             self.lib["public.glyphOrder"] = value
 
     @property
-    def guidelines(self) -> List[Guideline]:
+    def guidelines(self) -> list[Guideline]:
         """The font's global guidelines.
 
         Getter:
@@ -367,12 +362,16 @@ class Font:
         return self.info.guidelines
 
     @guidelines.setter
-    def guidelines(self, value: Iterable[Union[Guideline, Mapping[str, Any]]]) -> None:
+    def guidelines(self, value: Iterable[Guideline | Mapping[str, Any]]) -> None:
         self.info.guidelines = []
         for guideline in value:
             self.appendGuideline(guideline)
 
-    def objectLib(self, object: HasIdentifier) -> Dict[str, Any]:
+    kerning = property(_get_kerning, _set_kerning)
+
+    lib = property(_get_lib, _set_lib)
+
+    def objectLib(self, object: HasIdentifier) -> dict[str, Any]:
         """Return the lib for an object with an identifier, as stored in a font's lib.
 
         If the object does not yet have an identifier, a new one is assigned to it. If
@@ -394,27 +393,27 @@ class Font:
         return _object_lib(self.lib, object)
 
     @property
-    def path(self) -> Optional[PathLike]:
+    def path(self) -> PathLike | None:
         """Return the path of the UFO, if it was set, or None."""
         return self._path
 
     @property
-    def bounds(self) -> Optional[BoundingBox]:
+    def bounds(self) -> BoundingBox | None:
         """Returns the (xMin, yMin, xMax, yMax) bounding box of the default
         layer, taking the actual contours into account.
 
         |defcon_compat|
         """
-        return self.layers.defaultLayer.bounds
+        return self.layers._defaultLayer.bounds
 
     @property
-    def controlPointBounds(self) -> Optional[BoundingBox]:
+    def controlPointBounds(self) -> BoundingBox | None:
         """Returns the (xMin, yMin, xMax, yMax) bounding box of the layer,
         taking only the control points into account.
 
         |defcon_compat|
         """
-        return self.layers.defaultLayer.controlPointBounds
+        return self.layers._defaultLayer.controlPointBounds
 
     def addGlyph(self, glyph: Glyph) -> None:
         """Appends glyph object to the default layer unless its name is already
@@ -424,12 +423,12 @@ class Font:
             Call the method on the layer directly if you want to overwrite entries
             with the same name or append copies of the glyph.
         """
-        self.layers.defaultLayer.addGlyph(glyph)
+        self.layers._defaultLayer.addGlyph(glyph)
 
     def newGlyph(self, name: str) -> Glyph:
         """Creates and returns new :class:`.Glyph` object in default layer with
         name."""
-        return self.layers.defaultLayer.newGlyph(name)
+        return self.layers._defaultLayer.newGlyph(name)
 
     def newLayer(self, name: str, **kwargs: Any) -> Layer:
         """Creates and returns a new :class:`.Layer`.
@@ -449,7 +448,7 @@ class Font:
             overwrite: If False, raises exception if newName is already taken.
                 If True, overwrites (read: deletes) the old :class:`.Glyph` object.
         """
-        self.layers.defaultLayer.renameGlyph(name, newName, overwrite)
+        self.layers._defaultLayer.renameGlyph(name, newName, overwrite)
 
     def renameLayer(self, name: str, newName: str, overwrite: bool = False) -> None:
         """Renames a :class:`.Layer`.
@@ -462,7 +461,7 @@ class Font:
         """
         self.layers.renameLayer(name, newName, overwrite)
 
-    def appendGuideline(self, guideline: Union[Guideline, Mapping[str, Any]]) -> None:
+    def appendGuideline(self, guideline: Guideline | Mapping[str, Any]) -> None:
         """Appends a guideline to the list of the font's global guidelines.
 
         Creates the global guideline list unless it already exists.
@@ -482,7 +481,7 @@ class Font:
             self.info.guidelines = []
         self.info.guidelines.append(guideline)
 
-    def write(self, writer: UFOWriter, saveAs: Optional[bool] = None) -> None:
+    def write(self, writer: UFOWriter, saveAs: bool | None = None) -> None:
         """Writes this Font to a :class:`fontTools.ufoLib.UFOWriter`.
 
         Args:
@@ -494,7 +493,7 @@ class Font:
         if saveAs is None:
             saveAs = self._reader is not writer
         # TODO move this check to fontTools UFOWriter
-        if self.layers.defaultLayer.name != DEFAULT_LAYER_NAME:
+        if self.layers._defaultLayer.name != DEFAULT_LAYER_NAME:
             assert DEFAULT_LAYER_NAME not in self.layers.layerOrder
         # save font attrs
         writer.writeFeatures(self.features.text)
@@ -514,9 +513,9 @@ class Font:
 
     def save(
         self,
-        path: Optional[Union[PathLike, fs.base.FS]] = None,
+        path: PathLike | fs.base.FS | None = None,
         formatVersion: int = 3,
-        structure: Optional[UFOFileStructure] = None,
+        structure: UFOFileStructure | None = None,
         overwrite: bool = False,
         validate: bool = True,
     ) -> None:
